#!/usr/bin/env python
import sys, string, os, commands, tempfile, glob, re
sys.path.append("/opt/Embedix/lwiz1.0/usr/lib/python2.0/site-packages")

import wizutil
from messaging import errMsg, dbgMsg, stdMsg
#this is the mapping of debian packages to equivalent embedix packages
#for use when calculating dependencies

embedixDepMappings = {
"libc6" : ["glibc"],
"libncurses5" : ["keep-libncurses.so"],
"libreadline3" : ["keep-libreadline.so"],
"zlib1g" : ["libz"]
}


### always set up the path the way we want it
os.environ["PATH"] = "/opt/Embedix/emb-bin:/opt/Embedix/bin:" + os.environ["PATH"]


#this function will parse the argument list
def parseOptions(arg_list, option_dict):
    "Parse arguments from the commands line into dictionary form"
    #the option dictionary should look like this with keys and possible options
    #the "value_options" key lists the options that expect values directly after them

    #option_dict = {}
    #option_dict["help"] = ("-h", "--help")
    #option_dict["dir"] = ("-d", "--projdir", "--projectdir")
    #option_dict["lpf"] = ("-l", "--lpf")
    #option_dict["ecd"] = ("-e", "--ecd")
    #option_dict["tar"] = ("-t", "--tar")
    #option_dict["lbc"] = ("--lbc",)
    #option_dict["value_options"] = ("dir",)

    #list of options that take values after them
    value_options = option_dict["value_options"]
    arg_dict = {}
    status_flag = 1
    i = 0
    while i < len(arg_list):
        ate_arg = 0
        for key in option_dict.keys():
            arg = arg_list[i]
            if arg in option_dict[key]:
                ate_arg = 1 #eat (deal with) the arguments
                #if this argument is already in the dictionary then
                #tell the user
                if key in arg_dict.keys():
                    status_flag = 0
                    print "duplicate argument:", arg
                    break
                #if this option expects a following arg then look for it
                #and throw an error otherwise
                value = 1
                if key in value_options:
                    if i+1 < len(arg_list) and arg_list[i+1][0] != "-":
                        i = i + 1
                        value = arg_list[i]
                    else:
                        status_flag = 0
                        print "Error: no value given for option", arg
                arg_dict[key] = value
                break
        #if we never ate the argument then tell the user
        if ate_arg == 0:
            status_flag = 0
            print "Unrecognized option:", arg
        i = i + 1

    return (status_flag, arg_dict)


class EcdNode:
        #indent increment
        ind = "    "
        #allowable fields in ecd nodes.
        fields = ['NAME', 'NODETYPE', 'TYPE', 'DEFAULT_VALUE',
                'PROMPT', 'LICENSE', 'HELP', 'KEEPLIST',
                'MIN_DYNAMIC_SIZE', 'PROVIDES', 'BUILD_REQUIRES',
                'BUILD_VARS', 'REQUIRES', 'REQUIRESEXPR', 'SRPM',
                'SPECPATCH', 'STARTUP_TIME', 'STATIC_SIZE',
                'STORAGE_SIZE', 'VALUE', 'TRIDEPS', 'CHOICELIST',
                'IF', 'CONFLICTS', 'TRANSIENT', 'RANGE', 'BUILD_TIME']
        #these fields should always use the <field> syntax even if they're only
        #one liners
        multiLineFields = ["BUILD_VARS", "CHOICELIST", "LICENSE", "HELP", "KEEPLIST", "REQUIRESEXPR"]

        #name is obvious, nodetype is group, component, option, or autovar
        def __init__(self, nodeType, name):
                self.nodeType = nodeType.upper()
                self.name = name
                self.fieldDict = {"NAME":name, "NODETYPE":nodeType}
                self.children = []

        def setField(self, field, value):
                field = field.upper()
                if not field in self.fields:
                        print "EcdNode::SetField called with illegal field %s" % field
                        return
                self.fieldDict[field] = value

        def getField(self, field):
                field = field.upper()
                if not field in self.fields:
                        print "EcdNode::GetField called with illegal field %s" % field
                        return None
                return self.fieldDict.get(field, "")

        #add a ecd node as a child of this node
        def addChild(self, child):
                self.children.append(child)

        def getFieldStr(self, field, indent = ""):
                field = field.upper()
                text = self.getField(field)
                if not text:
                        return ""
                if text.strip().find("\n") != -1 or field in self.multiLineFields:
                        fieldStr = indent + "<" + field + ">\n" + indent
                        fieldStr += self.ind + string.join(text.split("\n"), "\n" + indent + self.ind)
                        fieldStr += "\n" + indent + "</" + field + ">" + "\n"
                else:
                        fieldStr = indent + field + "=" + text.strip() + "\n"
                return fieldStr


        #get the ecd text for this node (includes children)
        def getNodeStr(self, indent = ""):
                nodeStr = indent + "<" + self.nodeType + " " + self.name + ">\n"
                for field in self.fields:
                        if field in ["NAME", "NODETYPE"]:
                                continue
                        nodeStr += self.getFieldStr(field, indent + self.ind)
                for child in self.children:
                        nodeStr += child.getNodeStr(indent + self.ind)
                nodeStr += indent + "</" + self.nodeType + ">\n"
                return nodeStr






####### here are some utility functions ########

#given a script, execute and get the output and status
def getStatusOutput(script):
        tmpscr = tempfile.mktemp()
        open(tmpscr, "w").write("#! /bin/sh\n" + script + "\n")
        (status, output) = commands.getstatusoutput("/bin/sh " + tmpscr)
        os.system("rm -f " + tmpscr)

        return (status, output)

#join a bunch of dictionaries into one and return the result
def dictMerge(*dicts):
        d = {}
        for dict in dicts:
                d.update(dict)
        return d

#check to make sure that a given directory really is a SDK project directory
def isProjectDir(directory):
        if not os.path.isdir(directory):
                return 0
        if len(glob.glob(directory + "/*.epj")):
                return 1
        return 0


########### here is the real meat ###############

controlFields = ["package", "maintainer", "description", "section",
                 "md5sum", "depends", "filename", "priority", "installed-size",
                 "version", "architecture", "size", "suggests", "recommends",
                 "source", "replaces", "conflicts", "provides", "pre-depends",
                 "essential"]
#parse a debian control file and return a dictionary with each of
#the fields as keys
def parseControlFile(text):
        packageDict = {}
        for line in text.split("\n"):
                colonIndex = line.find(":")
                if colonIndex != -1 and not line[0] in " \t":
                        field = line[:colonIndex].lower()
                        text = line[colonIndex+1:]
                else:
                        text = line

                text = text.strip()
                if packageDict.has_key(field):
                        text = packageDict[field] + "\n" + text
                packageDict[field] = text
        return packageDict

#read all of the control texts in a debian Packages file and return a dictionary
#of dictionaries
def readPackagesFile(filename):
        packagesDict = {}
        for controlText in open(filename).read().split("\nPackage:"):
                controlText = "Package:" + controlText
                packageDict = parseControlFile(controlText)
                if packageDict.has_key("package"):
                        packagesDict[packageDict["package"]] = packageDict
        return packagesDict

#given a package name and a database as a dicionary of control file dictionaries,
#track the dependencies down.
def findPackageDependencies(name, database):
        deps = [name]
        

#this is a mapping of debian architecture names to sdk architecture names
archMappings = {
                "i386": "i386",
                "arm": "arm",
                "powerpc": "ppc",
                "m68k": "m68k",
                "mips": "mips",
                "mipsel": "mipsel",
                "all": "all"
                        }

#given a command and the debian architecture, find the correct binary in the 
#/o/e/tools/bin directory
def getArchBinary(command, arch):  
    arch = arch.strip()
    if arch == "all":
        return command        
    if archMappings.has_key(arch):
        archPrefix = "/opt/Embedix/tools/bin/" + archMappings[arch] + "-linux-"
        if os.path.exists(archPrefix + command):
            return archPrefix + command
        else:
            print "could not find command '%s' for architecture %s" % (command, arch)
            return command
    else:
        print "architecture %s is unknown" % arch
        return command
    
    



#this class encapsulates all kinds of information about a debian package
#it does the work of creating all kinds of files, gathering information
#calculating dependencies, etc.
class DebianBinaryPackage:
        #path to debian install scripts on target system
        debPath = "/var/lib/deb/info"
        #this is a list of all possible fields that can be found in the control file
        #of a debian binary package.
        controlFields = ["package", "maintainer", "description", "section",
                        "md5sum", "depends", "filename", "priority", "installed-size",
                        "version", "architecture", "size", "suggests", "recommends",
                        "source", "replaces", "conflicts", "provides", "pre-depends",
                        "essential"]

        #the fields that define the dependencies for a debian package in the control file
        #dependentFields = ["depends", "pre-depends", "suggests"] this is too liberal
        dependentFields = ["depends", "pre-depends"]





        helpStrings = {
                        "man": "Include the man pages for %s?",
                        "info": "Include the info pages for %s?",
                        "doc": "Include the documentation for %s?",
                        "locale": "Include the locale strings for %s?"
                        }

        def __init__(self, fileName):
                self.debug = 0
                self.lbcTemplate = "lbc.in" #FIXTHIS - should set this somewhere else
                self.makefileTemplate = "Makefile.in" #FIXTHIS - should set this somewhere else
                #the path contains the whole path to the file (including directories)
                self.filePath = fileName
                #the filename should be only the name of the file without directories
                fileName = os.path.basename(fileName)
                self.fileName = os.path.basename(fileName)

                #prime the control field dictionary with empty strings in all places
                self.fieldDict = {}
                for field in self.controlFields:
                        self.fieldDict[field] = ""


                # the build directory in the lbc will be everything up to the last
                # dash, in the debian filename or if there are no dashes then
                # everything up to the .deb extension
                parts = fileName.split("-")
                if len(parts) > 1:
                        self.buildDir = string.join(parts[:-1], "-")
                else:
                        self.buildDir = fileName.replace(".deb", "")
                #the source in the lbc tar file name will be the build directory + _deb.tgz
                self.tarFileName = self.buildDir + "_deb.tgz"

                #the normal name of gzip_1.2.3-fun_i386.deb should be gzip
                self.normalName = fileName.split("_")[0]
                #the name of the above should be gzip_deb to distinguish it from
                #a normal sdk package gzip
                self.name = self.normalName + "_deb"

                print "The name of this package is:", self.name

                self.unpacked = 0
                self.binaryFiles = []
                self.unpackPackage()
                self.gatherInformation()

        def setDebug(self, debug):
                self.debug = debug
                if debug:
                        print "deb2lpf: debugging turned on"

        #return the name with the _deb suffix
        def getName(self):
                return self.name

        #get a list of all of the binary files and parse the control file
        def gatherInformation(self):
                if not self.unpacked:
                        print "DebianBinaryPackage::GatherInformation called when not unpacked!"
                        return
                self.binaryFiles = self.getBinaryFiles()
                self.getControlText()

        #This will unpack the deb file and it's contents into a temporary directory
        def unpackPackage(self):
                self.dirPath = "/%s/%s" % (tempfile.mktemp(), self.name)
                fileName = self.fileName
                filepath = self.filePath
                (status, output) = getStatusOutput("""
set -e
set -x
if [ -d %(dirPath)s ] ; then rm -rf %(dirPath)s ; fi
mkdir -p %(dirPath)s
cp %(filePath)s %(dirPath)s
cd %(dirPath)s
ar -x %(fileName)s
mkdir control
(cd control; tar zxf ../control.tar.gz)
mkdir data
(cd data; tar zxf ../data.tar.gz)
                """ % self.__dict__)
                self.unpacked = 1

        def __del__(self):
                #when this object is deleted, we want to remove the temporary
                #directories that were used.
                if self.unpacked == 0:
                        return

                #if we're debugging then don't remove the temporary directories
                if self.debug:
                        print "not removing temporary directories located at %s" % self.dirPath
                        return

                tmpPath = os.path.dirname(self.dirPath)
                #print "dirpath is %s, tmp path is %s" % (self.dirPath, tmpPath)
                print "removing temporary directories...",
                (status, output) = getStatusOutput("\nrm -rf %s" % tmpPath)
                if status == 0:
                        print "success"
                else:
                        print "failed"
                        print "there may be remnant temporary directories located at %s" % self.dirPath

        def getBinaryFiles(self):
                #if we did this already, just return the old results
                if self.binaryFiles:
                        return self.binaryFiles
                #FIX-THIS probably should check the status
                (status, output) = getStatusOutput("""
set -e
cd %(dirPath)s
tar ztf data.tar.gz
                """ % self.__dict__)
                #print "Binary Files are:"
                #print output
                binFiles = output.split()
                return binFiles

        def getInstallFiles(self):
                installFiles = []
                for controlFile in ["postinst", "preinst"]:
                        if os.path.exists(self.dirPath + "/control/" + controlFile):
                                installFiles.append(self.debPath + "/" + self.name + "." + controlFile)
                return installFiles
                
                
        #given a file name with directories, return the text of the file                
        def getFileText(self, filepath):
            dbgMsg("checking", filepath)
            text = ""
            if os.path.exists(self.dirPath + "/" + filepath):
                dbgMsg(filepath, "exists")
                text = open(self.dirPath + "/" + filepath).read()
                dbgMsg("text is:\n" + text)
            return text
                
      

        #given a file list, this will determine what directories in the list
        #are empty.  Directories must have a trailing slash / to indicate them
        #as directories.
        def getEmptyDirectories(self, fileList):
                if not self.unpacked:
                        self.unpackPackage()
                dirs = []
                emptyDirs = []
                #make a list of all of the directories in the binary data
                for fileName in fileList:
                        if fileName[-1] == "/":
                                dirs.append(fileName)

                #now check each member of the filelist against each directory
                #if a file list has a commonprefix with a directory then don't
                #add the directory
                for directory in dirs:
                        empty = 1
                        for fileName in fileList:
                                if fileName == directory:
                                        continue
                                if fileName.find(directory) == 0:
                                        empty = 0
                                        break
                        if empty:
                                if directory[0] == ".":
                                        directory = directory[1:]
                                emptyDirs.append(directory)

                #print "emptry dirs are", emptyDirs
                return emptyDirs

        #given a dependency like foo | bar return (foo=="y" || bar=="y")
        #but given a normal dependency, just return foo=="y"
        def getDepStr(self, dep):
                depStr = ""
                deps = dep.split("|")
                deps = map(string.strip, deps)
                for dep in deps[:]:
                        if dep in embedixDepMappings.keys():
                                for embedixDep in embedixDepMappings[dep]:
                                        if not embedixDep in deps:
                                                deps.append(embedixDep)


                #if there's only one then do (dep == "y")
                if len(deps) == 1:
                        depStr += "(" + deps[0] + ' == "y")'
                else: #otherwise do a complex expression
                        depStr += "((" + deps[0] + ' == "y")'
                        for dep in deps[1:]:
                                depStr += " || (" + dep + ' == "y")'
                                depStr += ")"
                return depStr

        #extract dependency information from the control file and return the
        #type of node: requires or requiresexpr along with the string that describes
        #the dependencies.
        def getDependencyInfo(self):
                allDeps = ["debsupport"]
                depStr = ""
                for field in self.dependentFields:
                        deps = self.getControlField(field).split(",")
                        for dep in deps:
                                if deps[0] == "":
                                        break
                                allDeps.append(dep)


                #FIX-THIS - we're stripping off library comparisons like libc6(>=2.1.0)
                #so that it turns into libc6.  This should not occur, but sdk 2.4 won't work
                #without it!
                depType = "requiresexpr"
                #this re will remove lib version stuff like (>=2.1.0)
                libNumRe = re.compile("\([^)]*\)")
                for i in range(len(allDeps)):
                        if allDeps[i].find("(") != -1:
                                allDeps[i] = libNumRe.sub("", allDeps[i])
                        allDeps[i] = allDeps[i].strip()

                #for a requiresexpr we and (&&) them and deal with or's specially
                depStr = self.getDepStr(allDeps[0])
                if len(allDeps) > 1: #if there are more then put &&'s between them
                        for dep in allDeps[1:]:
                                depStr += " &&\n" + self.getDepStr(dep)

                depStr = depStr.strip()
                print "dependency type :%s, dependencies :%s" % (depType, allDeps)
                return (depType, depStr)




        #this will return a dictionary with keys "main", "man", "doc", "locale" and "info"
        def divideFiles(self, fileList):
                newFileList = []
                for fileName in fileList:
                    if fileName[0] == ".":
                        fileName = fileName[1:]
                    if fileName[0] != "/":
                        fileName = "/" + fileName
                    if fileName in ["/", "./"]:
                        continue
                    if fileName[0] != "/":
                        fileName = "/" + fileName 
                    newFileList.append(fileName)
                    
                mainFiles = self.getEmptyDirectories(newFileList)
                manFiles = []
                docFiles = []
                infoFiles = []
                localeFiles = []
                for fileName in newFileList:
                        fileName = fileName.strip()
                        dbgMsg("looking at file", fileName)
                        #don't include directories
                        if fileName[-1] == "/":
                                continue
                        if fileName[0] == ".":
                                fileName = fileName[1:]
                        if fileName[0] != "/":
                                fileName = "/" + fileName
                        if fileName in ["/", "./"]:
                                continue
                                
                        #dbgMsg("not continuing")
                        
                                                                        
                        if fileName.find("/doc/") > -1:
                                docFiles.append(fileName)
                        elif fileName.find("/man/") > -1:
                                manFiles.append(fileName)
                        elif fileName.find("/info/") > -1:
                                infoFiles.append(fileName)
                        elif fileName.find("/locale/") > -1:
                                localeFiles.append(fileName)
                        else:
                                dbgMsg("adding file to main", fileName)
                                mainFiles.append(fileName)
                                dbgMsg("main files is", mainFiles)
                dbgMsg("dict", {"main":mainFiles, "man":manFiles, "doc":docFiles, "info":infoFiles, "locale":localeFiles})
                return {"main":mainFiles, "man":manFiles, "doc":docFiles, "info":infoFiles, "locale":localeFiles}


        #given a list of files, return the total storage and static sizes
        def getFileListSizes(self, fileList):
                storageSize = 0
                staticSize = 0
                sizeCmd = getArchBinary("size", self.getControlField("architecture"))
                if not self.unpacked:
                        self.unpackPackage()
                for fileName in fileList:
                        filePath = self.dirPath + "/data/" + fileName

                        #skip things like devices, symlinks, etc.
                        if not os.path.isfile(filePath) and not os.path.isdir(filePath):
                                continue

                        #get the filesize from the stat call
                        fileStorageSize = os.stat(filePath)[6]
                        storageSize += fileStorageSize

                        (status, output) = getStatusOutput(sizeCmd + " " + filePath)
                        if status == 0:
                                fileStaticSize = int(output.split("\n")[1].split()[3])
                                staticSize += fileStaticSize
                                #print "output of size is:\n" + output
                                #print "static size of %s is %i" % (fileName, fileStaticSize)
                return (storageSize, staticSize)


        #determine what services this package provides.  Basically this is all
        #libraries, binaries, and the name of the package
        def getProvidesList(self):
                #prime the list with the normal name of the package
                providesList = [self.normalName]
                for package in self.getControlField("provides").split(","):
                        providesList.append(package.strip())
                #now we want to include all binaries and libraries in the provides
                #list.
                for fileName in self.getBinaryFiles():
                        if fileName[-1] == "/":
                                continue
                        baseName = os.path.basename(fileName)
                        if baseName.find("lib") != -1:
                                parts = baseName.split(".")
                                for i in range(0, len(parts)):
                                        libName = string.join(parts[0:i+1], ".")
                                        if libName not in providesList:
                                                providesList.append(libName)

                        if fileName.find("bin") != -1:
                                for binPath in ["/bin/", "/sbin/", "/usr/bin", "/usr/sbin",
                                                "/usr/local/bin", "/usr/local/sbin"]:
                                        if fileName.find(binPath) != -1:
                                                basename = os.path.basename(fileName)
                                                if basename not in providesList:
                                                        providesList.append(basename)
                                                break

                providesList.sort()
                return providesList

        #open the control file, update our internal control field dictionary, and return
        #the text of the control file
        def getControlText(self):
                text = open(self.dirPath + "/control/control").read()
                #print "Control text is:"
                #print text
                self.fieldDict.update(parseControlFile(text))
                return text

        def getControlFileName(self):
                return self.name + ".ctrl"

        #write the control file to the given directory
        def writeControlFile(self, fdir="."):
                fileName = os.path.join(fdir, self.getControlFileName())
                open(fileName, "w").write(self.getControlText())


        def getControlField(self, field):
                if not field in self.controlFields:
                        print "DebianBinaryPackage::getControlField: field %s is not a control field" % field
                return self.fieldDict.get(field, "")

        #calculate the filesizes from the node's keeplist and set it's size fields
        #accordingly
        def calculateNodeSizes(self, node):
                print "Checking file sizes for node: %s" % node.getField("name")
                keeplist = node.getField("keeplist")
                if len(keeplist):
                        fileList = keeplist.split("\n")
                        (storageSize, staticSize) = self.getFileListSizes(fileList)
                        node.setField("static_size", str(staticSize))
                        node.setField("storage_size", str(storageSize))

        #get the SDK architecture string for this package.  It uses a mapping
        #of debian architecture strings to sdk architecture strings.  The debian arch
        #is found in the "architecture" field of the control file
        def getSDKArch(self):
                debArch = self.getControlField("architecture")
                if not debArch in archMappings.keys():
                        print "DebianBinaryPackage::getSDKArch: no available architecture mapping for %s" % debArch
                        return "BAD_ARCHITECTURE"
                return archMappings[debArch]


        #from the information we've gathered about this debian package,
        #create an appropriate ecd string
        #The group will always be Debian Packages.  Create options for the binary,
        #info pages, man pages, locale files, and documentation
        def getEcdStr(self, options = ["man", "info", "doc", "locale"], visibleName = None):
                fileDict = self.divideFiles(self.getBinaryFiles())
                if visibleName == None or visibleName.strip() == "":
                    visibleName = self.normalName
                g = EcdNode("group", "Debian Packages")
                c = EcdNode("component", self.name)
                c.setField("help", self.fieldDict["description"])
                c.setField("prompt", visibleName)
                c.setField("requires", "include_" + self.name)
                g.addChild(c)
                mainOption = EcdNode("option", "include_" + self.name)
                mainOption.setField("prompt", "Include Debian " + visibleName)
                mainOption.setField("default_value", "1")
                keeplist = string.join(fileDict["main"] + self.getInstallFiles(), "\n")
                mainOption.setField("keeplist", keeplist)
                self.calculateNodeSizes(mainOption)
                (requiresField, requiresString) = self.getDependencyInfo()
                mainOption.setField(requiresField, requiresString)
                mainOption.setField("build_vars", "DEBARCH=" + self.getSDKArch())
                mainOption.setField("provides", string.join(self.getProvidesList(), "\n"))
                c.addChild(mainOption)
                for docType in options:
                        if len(fileDict[docType]):
                                option = EcdNode("option", "include_" + self.name + "_" + docType)
                                option.setField("prompt", self.helpStrings[docType] % visibleName)
                                option.setField("keeplist", string.join(fileDict[docType], "\n"))
                                self.calculateNodeSizes(option)
                                option.setField("default_value", "0")
                                option.setField("build_vars", "INCLUDE_" + docType.upper() + "=1")
                                c.addChild(option)
                return g.getNodeStr()

        def writeEcdFile(self, fdir = ".", options = ["man", "info", "doc", "info"], visibleName = None):
                fileName = os.path.join(fdir, self.getEcdFileName())
                open(fileName, "w").write(self.getEcdStr(options, visibleName))

        def getEcdFileName(self):
                return self.name + ".ecd"

        def getLBCStr(self):
                #read in the text of the template and replace the
                #appropriate strings
                #lbcStr = open(self.lbcTemplate).read()
                lbcStr = lbc_in
                lbcStr = lbcStr.replace("@TARFILE@", self.tarFileName)
                lbcStr = lbcStr.replace("@BUILDDIR@", self.buildDir)
                return lbcStr

        def writeLBCFile(self, fdir = "."):
                fileName = os.path.join(fdir, self.getLBCFileName())
                open(fileName, "w").write(self.getLBCStr())

        def getLBCFileName(self):
                return self.name.replace("-", "_") + ".lbc"

        def getMakefileStr(self):
                #read in the text of the template and replace the
                #appropriate strings
                #mfStr = open(self.makefileTemplate).read()
                mfStr = makefile_in
                mfStr = mfStr.replace("@DEBFILE@", self.fileName)
                mfStr = mfStr.replace("@DEBNAME@", self.name)
                return mfStr

        def getTarFileName(self):
                return self.tarFileName

        def makeTarFile(self, fdir = ".", preInstallText=None, postInstallText=None):
                tempDir = tempfile.mktemp()
                dirPath = "/%s/%s" % (tempDir, self.buildDir)
                debFile = self.filePath
                tarFileName = self.tarFileName
                buildDir = self.buildDir

                command = """
set -e
set -x
mkdir -p %(dirPath)s
cd %(dirPath)s
ar -x %(debFile)s
mkdir control
(cd control; tar zxf ../control.tar.gz)
mkdir data
(cd data; suwrapper tar zpsx --same-owner -f ../data.tar.gz)
rm -f data.tar.gz
rm -f control.tar.gz
""" % locals()
                (status, output) = getStatusOutput(command)
                
                dbgMsg("output is\n" + output)
                
                if status:
                    errMsg("command failed, output was\n" + output)

                #modify the pre and post install files
                #to include the following after the first line
                #INSTALL_DOC=@INSTALL_DOC@
                #INSTALL_MAN=@INSTALL_MAN@
                #the @foo@ will be replaced by a 1 or 0 at make install time by the make file
                #so the script should be modified to use these values
                if preInstallText == None and postInstallText == None:
                    for installFile in ["preinst", "postinst", "prerm", "postrm"]:
                            if not os.path.exists(dirPath + "/control/" + installFile):
                                    continue
                            lines = open(dirPath + "/control/" + installFile).readlines()
                            newInstallFile = open(dirPath + "/control/" + installFile, "w")
                            newInstallFile.write(lines[0])
                            for buildVar in ["INCLUDE_DOC", "INCLUDE_MAN", "INCLUDE_LOCALE", "INCLUDE_INFO"]:
                                    newInstallFile.write(buildVar + "=@" + buildVar + "@\n")
                            newInstallFile.write(string.join(lines[1:], "") + "\n")
                            
                if preInstallText != None and preInstallText.strip():
                    open(dirPath + "/control/preinst", "w").write(preInstallText)
                if postInstallText != None and postInstallText.strip():
                    open(dirPath + "/control/postinst", "w").write(postInstallText)

                #write the makefile
                open(os.path.join(dirPath, "Makefile"), "w").write(self.getMakefileStr())

                #build the tarball and remove the directory
                command = """
(cd %(tempDir)s;tar zcf %(tarFileName)s %(buildDir)s)
mv %(tempDir)s/%(tarFileName)s %(fdir)s """ % locals()
                (status, output) = getStatusOutput(command)
                
                print "removing tar file directories...",
                (status, output) = getStatusOutput("\nsuwrapper rm -rf %s" % tempDir)
                if status == 0:
                        print "success"
                        return 1
                else:
                        print "failed"
                        print "there may be remnant temporary directories located at %s" % tempDir
                        return 0

        def installIntoProject(self, projDir):
                if not isProjectDir(projDir):
                        print "deb2lpf: %s is not an Embedix SDK project directory" % projDir
                        return 0
                    
                debArch = archMappings.get(self.getControlField("architecture"), "unknown")
                projectArch = wizutil.parse_config(projDir + "/config-data/buildcontrol/board/bsp_config")["arch"].strip()
                if not (debArch == "all" or debArch == projectArch):
                    print "This debian file is for the %s architecture and the project is for the %s architecture.  Quitting." % (debArch, projectArch)
                    return 0
                #first create the lbc and put it into <project>/config-data/buildcontrol/local
                lbcDir = os.path.join(projDir, "config-data/buildcontrol/local/")
                print "Installing %s into %s" % (self.getLBCFileName(), lbcDir)
                self.writeLBCFile(lbcDir)

                #next create the ecd and put it into <project>/config-data/ecds/local
                ecdDir = os.path.join(projDir, "config-data/ecds/local/")
                print "Installing %s into %s" % (self.getEcdFileName(), ecdDir)
                self.writeEcdFile(ecdDir)

                #now drop the tar file into <project>/Packages/local
                tarDir = os.path.join(projDir, "Packages/local/")
                print "Installing %s into %s" % (self.getTarFileName(), tarDir)
                self.makeTarFile(tarDir)         

                print """
The debian package %(fileName)s has been installed into the project located
at %(projDir)s.  The package will be found in the toplevel group labeled
"Debian Packages".  Please make sure that you have also installed the
depsupport package or this package may not be configure properly on the target.
""" % dictMerge(locals(), self.__dict__)
                return 1
            




def printOptions():
        optionText = """
Usage: deb2lpf [OPTION]... DEBIAN_FILE
Create Embedix SDK package files for the given DEBIAN_FILE.

  -c, --control         extract the control file from DEBIAN_FILE
  -d DIR                create and place all of the needed files (ecd, lbc, and tarball)
      --projdir DIR     into the project located in this directory.  This must
                        be a TW project directory.

  -e, --ecd             create the ecd from DEBIAN_FILE
  -h, --help            show this screen
  -l, --lpf             create a complete Embedix SDK package from DEBIAN_FILE
      --lbc             create the lbc from DEBIAN_FILE
  -t, --tar             create the lbc from DEBIAN_FILE

Any combination of options may be given provided that each option
be given at most only once.

DEBIAN_FILE should be a normal binary debian package of the form
<name>[_<version>]-<debian release>.deb an example would be:
foo_1.0-12.deb

if the following command were given:

deb2lpf -d /home/chrisb/project/intel --lpf --lbc -e -t foo_1.0-12.deb

then foo_deb.lbc, foo_deb.ecd, and foo_1.0-12deb.tgz would be created and
placed in the directory that the script was invoked from.  This will be done
per the --lbc, -e, and -t options.  These files would also be placed in the
correct locations under the project directory /home/chrisb/project/intel per the
-d option. Finally a file foo_deb-1.0-1.lpf will be created and placed in the
directory that the script was invoked from per the --lpf option.

report bugs to <chrisb@lineo.com>

"""
        optionText = optionText.strip() + "\n"
        print optionText

optionDict = {}
optionDict["help"] = ("-h", "--help")
optionDict["dir"] = ("-d", "--projdir", "--projectdir")
optionDict["lpf"] = ("-l", "--lpf")
optionDict["ecd"] = ("-e", "--ecd")
optionDict["tar"] = ("-t", "--tar")
optionDict["lbc"] = ("--lbc",)
optionDict["control"] = ("-c", "--control")
optionDict["debug"] = ("--debug",)
optionDict["value_options"] = ("dir",)


def errorMessage(message):
        sys.stderr.write("deb2lpf: " + message + "\n")
        sys.stderr.write("try 'deb2lpf --help' for more information\n")


def checkStartupCommandLine():
        if len(sys.argv) < 2:
                errorMessage("must specify at least one option and the deb file")
                return (0, None)
        program, optionList, debFile = sys.argv[0], sys.argv[1:-1], sys.argv[-1]
        (status, options) = parseOptions(optionList, optionDict)
        if "-h" in sys.argv or "--help" in sys.argv:
                printOptions()
                return (0, None)
        if debFile[0] == "-":
                errorMessage("no deb file given as last argument")
                return (0, None)
        if not os.path.exists(debFile):
                errorMessage("could not locate file %s" % debFile)
                return (0, None)
        if debFile[-4:] != ".deb":
                errorMessage("deb file %s does not look like a real debian package file" % debFile)
                return (0, None)
        if status == 0:
                errorMessage("bad options")
                return (0, None)
        options["deb"] = debFile
        #print "options", options
        return (1, options)


def run():
        (status, options) = checkStartupCommandLine()
        if status == 0:
                sys.exit(1)
        dbp = DebianBinaryPackage(options["deb"])

        global debDict, file2node

        #print "loading debian information"
        #debDict = cPickle.Unpickler(open("debinfo.db")).load()
        #print "loading project information"
        #file2node = cPickle.Unpickler(open("projinfo.db")).load()

        if options.has_key("debug"):
                dbp.setDebug(1)
        if options.has_key("control"):
                print "Writing ecd file: %s" % dbp.getControlFileName()
                try:
                        dbp.writeControlFile()
                except Exception, e:
                        print "Error while writing ecd file %s : %s" % (dbp.getControlFileName(), e)
        if options.has_key("ecd"):
                print "Writing ecd file: %s" % dbp.getEcdFileName()
                try:
                        dbp.writeEcdFile()
                except Exception, e:
                        print "Error while writing ecd file %s : %s" % (dbp.getEcdFileName(), e)
        if options.has_key("lbc"):
                print "Writing lbc file: %s" % dbp.getLBCFileName()
                try:
                        dbp.writeLBCFile()
                except Exception, e:
                        print "Error while writing lbc file %s : %s" % (dbp.getLBCFileName(), e)
        if options.has_key("tar"):
                print "Making tar file: %s" % dbp.getTarFileName()
                try:
                        dbp.makeTarFile()
                except Exception, e:
                        print "Error while making tar file %s : %s" % (dbp.getTarFileName(), e)
        if options.has_key("dir"):
            
                print "Installing package %s into project located at %s" % (dbp.getName(), options["dir"])
                dbp.installIntoProject(options["dir"])



makefile_in = """
# This Makefile was created automatically for use with a debian
# binary package.

prefix=/

DEBFILE=@DEBFILE@
DEBNAME=@DEBNAME@
DEBDIR=$(prefix)/var/lib/deb/info

all: build

#for the build stage, just check the architectures
build: checkarch

#check the $ARCH variable and match it against the $DEBARCH variable
#which should be exported by all debian packages.
#basically if there is an $ARCH var and a $DEBARCH var and they
#don't match then throw an error
checkarch:
\t@echo "checking architectures"
\t@if [ -n "${ARCH}" -a -n "${DEBARCH}" -a "${ARCH}" != "${DEBARCH}" -a "${DEBARCH}" != "all" ] ; \\
\tthen echo "project architecture is ${ARCH} and the debian package architecture is ${DEBARCH}!"; \\
\techo "Quitting!"; false; else echo "project architecture matches debian package architecture"; fi

#do a recursive copy of the data directory to the
#prefix root directory
install: build
\tcp -a data/* $(prefix)/
\tinstall -g root -o root -d $(DEBDIR)
#replace all @foo@ strings in install/remove scripts with a "1" or just "" depending on the environment
\tfor INSTALL_FILE in preinst postinst prerm postrm ;\\
\tdo \\
\t\tif [ -e control/$${INSTALL_FILE} ] ; then \\
\t\t\tsed -e s/@INCLUDE_DOC@/\"$${INCLUDE_DOC}\"/ control/$${INSTALL_FILE} | \\
\t\t\tsed -e s/@INCLUDE_MAN@/\"$${INCLUDE_MAN}\"/ | \\
\t\t\tsed -e s/@INCLUDE_INFO@/\"$${INCLUDE_INFO}\"/ | \\
\t\t\tsed -e s/@INCLUDE_LOCALE@/\"$${INCLUDE_LOCALE}\"/ > $(DEBDIR)/$(DEBNAME).$${INSTALL_FILE} ; \\
\t\t\tchmod a+rx $(DEBDIR)/$(DEBNAME).$${INSTALL_FILE}; \\
\t\tfi ; \\
\tdone

#get rid of everything but the Makefile and the debfile
clean:
\t@echo "Everything is clean, nothing to do"
"""

lbc_in = """
%pkg_file
@TARFILE@

%bld_dir_name
@BUILDDIR@

%makec
if [ -s configure ] ; then ./configure ; fi

%makeb
make

%makei
make install prefix=$BUILD_ROOT

%makedc
make clean
"""
                

if __name__ == "__main__":
        run()

