#!env python
#
# ecdlib.py - a library of ecd management routines
# this is essentially a python version of tigger
#
# todo:
#    read ECD file into data model in memory
#
# bugs:
#    
# $Id: ecdlib.py,v 1.1 2002/07/08 18:08:44 chrisb Exp $

import sys
import os
import string
import copy

####################################################################
#    lexical analyzing (low-level parsing) routines
####################################################################

# convert the items in the file to a token structure
# then write out the ecd file from the structure

# tokens:
# 


container_tags=["option", "group", "component", "autovar"]

attribute_tags=["default_value", "help", "keeplist", "min_dynamic_size",
        "prompt", "provides", "build_requires", "build_vars",
        "requires", "requiresexpr", "srpm", "specpatch",
        "startup_time", "static_size", "storage_size", "type",
        "value", "trideps", "choicelist", "if", "conflicts",
        "transient", "range", "build_time", "license"]

keylist= container_tags + attribute_tags +  \
    [ "<", ">", "=",
    "bool", "int", "int.hex", "string", "tristate" ]

class token_c:
    def __init__(self, type, value='', whitespace=''):
        self.type = type
        self.value = value
        self.whitespace = whitespace

# reads lines from a file.  It strips comments, and combines lines
# based on continuation characters
def read_file(filename):
    file = open(filename)
    lines = []
    line = file.readline()
    combine = 0
    while line:
        # strip leading and trailing whitespace
        line = string.strip(line)

        # skip empty lines
        if len(line)==0:
            line = file.readline()
            continue

        # ignore comment lines
        if line[0]=='#':
            line = file.readline()
            continue

        # combine this line with next one
        if not combine:
            # add line to list
            lines.append(line)
        else:
            lines[-1] = lines[-1] + line

        # check to see if next line should be combined
        if lines[-1][-1]=="\\":
            # remove trailing slash, and set flag
            lines[-1] = lines[-1][:-1]
            combine = 1
        else:
            combine = 0

        line = file.readline()

    file.close()
    return lines

# the following variables are used by the tokenizer
# lines = current list of lines from the file
# curline = current index into lines array
# toklist = list of tokens from the current line
# curtok = current index into lines array
def readchar():
    global lines, curpos, curline

    if curline > len(lines)-1:
        return None

    curpos = curpos + 1
    if curpos > len(lines[curline])-1:
        curpos = -1
        curline = curline + 1
        return "\n"

    char = lines[curline][curpos]
    #print "char=", char
    return char

def read_string(terminator):
    char = readchar()
    string = ""
    # eat leading whitespace
    ## while char and char in " \t":
        ## char = readchar()
    # now accumulate string
    while char and char != terminator: ## and char != "\n":
        string = string+char
        char = readchar()
    return string

def get_line():
    global verbos, toklist
    while len(toklist):
        token = toklist[-1]
        del toklist[-1]
        if verbose:
            pass

def get_token():
    global verbose, toklist

    # see if there's anything on the pushed token stack
    if len(toklist):
        token = toklist[-1]
        del toklist[-1]
        if verbose:
            print "returning pushed token"
            print_token(token)
        return token

    char = readchar()
    if not char:
        return None

    token_string = ""
    whitespace = ""

    ## eat leading whitespace
    while char in " \t":
        whitespace = whitespace+char
        char = readchar()

    if char=="'":
        token_string = "'" + read_string("'") + "'"
        token_type = "literal"
    if char=='"':
        token_string = '"' + read_string('"') + '"'
        token_type = "literal"
    if char=='(':
        token_string = '('+read_string(')')+')'
        token_type = "literal"

    if char=='\n':
        token_string = char
        token_type = "end-of-line"

    if char in '<>=':
        token_string = char
        token_type = "delimiter"

    if not token_string:
        while char not in " \t\n<>=":
            token_string = token_string+char
            char = readchar()
        if string.lower(token_string) in keylist:
            token_string = string.lower(token_string)
            token_type = "keyword"
        else:
            token_type = "word"

        # if last char read was EOL, set up to return EOL token
        # next time through
        if char=='\n':
            push_token(token_c("end-of-line", '\n', whitespace))

        # if last char read was <, set up to return < token
        # next time through
        if char in "<>=":
            push_token(token_c("delimiter", char, whitespace))

    token = token_c(token_type, token_string, whitespace)
    if verbose:
        print_token(token)
    return(token)

def push_token(token):
    global verbose, toklist
    #if verbose:
    #    print "pushing token=", token.value
    toklist.append(token)

####################################################################
#    node_c class definition
#    (including writing routines)
####################################################################

# routine to convert a nested ifstack expression into
# a single-line C-style expression, with parens
def convert_expr(expr):
    if expr[0]=='or':
        return convert_expr(expr[1])+' || '+convert_expr(expr[2])
    if expr[0]=='and':
        return convert_expr(expr[1])+' && '+convert_expr(expr[2])

    # convert single = to == (because it's more 'C'-like)
    if expr[1]=='=':
        middle='=='
    else:
        middle='!='
    # remove leading $ sign from first operand, and return
    # infix version of expression string
    return expr[0][1:]+' '+middle+' '+'"'+expr[2]+'"'

def de_space(str):
    try:
        sindex = string.index(str, ' ')
    except:
        sindex = -1
    try:
        tindex = string.index(str, '\t')
    except:
        tindex = -1
    if sindex != -1 or tindex != -1:
        newstr = ''
        for char in str:
            if char in ' \t/':
                char = '_'
            if char in '()':
                continue
            newstr = newstr+char
        return newstr
    else:
        return str

# this routine converts the ifstack (and possibly default value) into
# a string version of the expression
def make_ifstring(node):
    firstline = 1
    for expr in node.ifstack:
        try:
            expr = convert_expr(expr)
        except:
            print "cannot convert expr", expr
            print "in node", node.name
        if firstline:
            ifstring = "( %s )\n" % expr
        else:
            ifstring = ifstring + "&& ( %s )\n" % expr
        firstline = 0

    #remove trailing \n
    ifstring = ifstring[:-1]

    if node.nodetype == 'autovar':
        ifstring = "( "+ifstring+" )\n"
        ifstring = ifstring + 'LET ( $VALUE = "%s" )' % node.default
    return ifstring


# possible attributes of this node are:
# type, name, parent, childlist, prompt, help
class node_c:
    #this list affects the order that nodes will be written out in
    attribute_tags=["type", "default_value", "prompt", "license", "help", "keeplist", 
        "min_dynamic_size", "provides", "build_requires", "build_vars",
        "requires", "requiresexpr", "srpm", "specpatch",
        "startup_time", "static_size", "storage_size",
        "value", "trideps", "choicelist", "if", "conflicts",
        "transient", "range", "build_time", ]
    #this is a list of tags that will be not be written out with tag=value format even
    #if the value is only one line
    multi_line_tags=["build_vars", "keeplist"]
    # TRANSIENT can be: 1, 0
    # TYPE can be: int.hex, int.bool, int.dec, bool, string, choice, tristate
    # NODETYPE can be: GROUP, COMPONENT, OPTION, AUTOVAR
    #

    def __init__(self, nodetype, name, parent=None, ifstack=None):
        global nodemap

        self.childlist = []
        self.nodetype = nodetype
        self.name = name
        self.parent = parent
        if ifstack:
            self.ifstack = copy.deepcopy(ifstack)
        try:
            nodemap[name]=self
        except:
            pass

    def write_attr(self, file, attr, indent = ""):
        #see if this node really has this attribute
        if not hasattr(self, attr):
            return
        #get the attribute and see if it's splitable
        value = str(getattr(self, attr)).strip()
        value_lines = value.split("\n")
        if len(value_lines) > 1 or attr in self.multi_line_tags:
            file.write(indent + "<" + attr.upper() + ">\n")
            for line in value_lines:
                file.write(indent + "    "+ line.strip() + "\n")
            file.write(indent + "</" + attr.upper() + ">\n")
        else:
            file.write(indent + "" + attr.upper() + "=" + value + "\n")

    def write_node(self, file, indent=""):
        self.nodetype = self.nodetype.upper()
        if not self.nodetype.upper() in ["GROUP", "COMPONENT", "OPTION", "AUTOVAR"]:
            print "bad node type %s" % self.nodetype
            return

        file.write("%s<%s %s>\n" % (indent, self.nodetype, self.name))
        for attr in self.attribute_tags:
            if hasattr(self, attr) and str(getattr(self, attr)).strip():
                self.write_attr(file, attr, indent + "    ")
        if hasattr(self, "childlist"):
            for child in self.childlist:
                child.write_node(file, indent + "    ")
        file.write("%s</%s>\n" % (indent, self.nodetype))


    def write_section(self, file, indent=""):
        if self.nodetype.upper() == 'COMPONENT':
            file.write(indent+"<COMPONENT %s>\n" % self.name)
            # write other attributes as well
            if hasattr(self, 'srpm'):
                file.write(indent+"  SRPM=%s\n" % self.srpm)


            if hasattr(self, 'childlist'):
                for child in self.childlist:
                    child.write_section(file,
                            indent+"    ")
            file.write(indent+"</COMPONENT>\n")
        if self.nodetype.upper() == 'GROUP':
            file.write(indent+"<GROUP %s>\n" % self.name)
            # write other attributes as well
            if hasattr(self, 'prompt'):
                file.write(indent+"  PROMPT=%s\n" % self.prompt)
            if hasattr(self, 'default'):
                file.write(indent+"  DEFAULT_VALUE=%s\n"
                    % self.default)
            if hasattr(self, 'childlist'):
                for child in self.childlist:
                    child.write_section(file,
                            indent+"    ")
            file.write(indent+"</GROUP>\n")
        if self.nodetype.upper() == 'FAKEGROUP':
            # just write children (enclosing group should have
            # been handed in as a prefix)
            if hasattr(self, 'childlist'):
                for child in self.childlist:
                    child.write_section(file,
                            indent)
        if self.nodetype.upper() == 'OPTION' or self.nodetype.upper() =='AUTOVAR':
            if self.nodetype.upper() =='OPTION':
                file.write(indent+"<OPTION %s>\n" % self.name)
            else:
                file.write(indent+"<AUTOVAR %s>\n" % self.name)
            # write other attributes as well
            if hasattr(self, 'opttype'):
                # convert some kernel opttypes to ECD types
                if self.opttype =='hex':
                    # eventually change this to "int.hex"
                    file.write(indent+"  TYPE=int.hex\n")
                elif self.opttype =='choice':
                    file.write(indent+"  TYPE=string\n")
                else:
                    file.write(indent+"  TYPE=%s\n" % self.opttype)
            if hasattr(self, 'default'):
                if self.opttype =='hex':
                    file.write(indent+"  DEFAULT_VALUE=0x%s\n" % self.default)
                else:
                    file.write(indent+"  DEFAULT_VALUE=%s\n" % self.default)
            if hasattr(self, 'prompt'):
                file.write(indent+"  PROMPT=%s\n" % self.prompt)
            #if hasattr(self, 'prompt2'):
            #    file.write(indent+"  HINT=%s\n" % self.prompt2)

            if hasattr(self, 'help'):
                file.write(indent+"  <HELP>\n")
                for line in string.split(self.help, "\n"):
                    file.write(indent+" "+line+'\n')
                file.write(indent+"  </HELP>\n")

            if hasattr(self, 'trideps'):
                file.write(indent+"  TRIDEPS=%s\n" % self.trideps)
            if hasattr(self, 'choicelist'):
                file.write(indent+"  <CHOICELIST>\n")
                list = string.split(self.choicelist)
                for i in range(0,len(list),2):
                    file.write(indent+"    %15s %15s\n" % (list[i], list[i+1]))
                file.write(indent+"  </CHOICELIST>\n")

            if hasattr(self, 'static_size'):
                file.write(indent+"  STATIC_SIZE=%s\n" %
                    self.static_size)
            if hasattr(self, 'storage_size'):
                file.write(indent+"  STORAGE_SIZE=%s\n" %
                    self.storage_size)
            # convert expression list into string
            if hasattr(self, 'ifstack') and \
                not hasattr(self, 'ifstring'):
                self.ifstring = make_ifstring(self)

            if hasattr(self, 'requiresexpr'):
                file.write(indent+"  <REQUIRESEXPR>\n")
                for line in string.split(self.requiresexpr, "\n"):
                    file.write(indent+"   "+line+'\n')
                file.write(indent+"  </REQUIRESEXPR>\n")

            if hasattr(self, 'ifstring'):
                if self.type =='option':
                    file.write(indent+"  <REQUIRESEXPR>\n")
                else:
                    file.write(indent+"  <IF>\n")

                # write out each line of the ifstring, indented
                for line in string.split(self.ifstring, '\n'):
                    file.write(indent+"    %s\n" % line)

                if self.type == 'option':
                    file.write(indent+"  </REQUIRESEXPR>\n")
                else:
                    file.write(indent+"  </IF>\n")

            # now write out the choicelist autovar variables
            # these variables are of the form:
            # <AUTOVAR CONFIG_M386>
            #    TYPE=bool
            if hasattr(self, 'choicelist'):
                list = string.split(self.choicelist)
                for i in range(0,len(list),2):
                    opt_str = list[i]
                    choice_var = list[i+1]
                    file.write(indent+"  <AUTOVAR %s>\n" % choice_var)
                    file.write(indent+"    TYPE=bool\n")
                    file.write(indent+"    <IF>\n")
                    file.write(indent+'    (( %s == "%s" ) \n' % (self.name, choice_var))
                    file.write(indent+'    LET ( $VALUE = "y" ))\n')
                    file.write(indent+"    </IF>\n")
                    file.write(indent+"  </AUTOVAR>\n")

            if hasattr(self, 'childlist'):
                for child in self.childlist:
                    if child:
                        child.write_section(file,
                            indent+"    ")

            if self.nodetype =='option':
                file.write(indent+"</OPTION>\n")
            else:
                file.write(indent+"</AUTOVAR>\n")

####################################################################
#    high-level parsing (token processing)
####################################################################

def print_token(token):
    # make it so value prints nicer in the debug output
    if token.value == '\n':
        value = "\\n"
    else:
        value = token.value
    print "token=<",token.type, ",", value, ", |" + token.whitespace + "| >"

def get_multiline_value(attr_tag):
    global verbose

    # FIXTHIS - should read to closing tag directly, bypassing get_token
    # (the current algorithm may alter whitespace)
    valstr = ''
    c = readchar()
    done = 0
    while not done :
        if c != '<':
            valstr = valstr+c
        else:
            # check for matching close tag
            global toklist
            token = get_token()
            if token.value[0] != '/':
                ## found section with enclosed '<'?
                #print "in get_multiline_value("+attr_tag+")"
                valstr = valstr + '<' + token.value
                #print "valstr=", valstr
                while len(toklist):
                    token = get_token()
                    valstr = valstr + token.value
            else:
                # token starts with '/'
                if string.lower(token.value)[1:] != attr_tag:
                    raise SyntaxError, "mismatched attribute tag: %s != %s" % (attr_tag, token.value)

                t2 = get_token()    # read '>'
                if t2.value != '>':
                    valstr = valstr + '<' + token.value + t2.value
                #t3 = get_token()    # read eol
                #if t3.value != '\n':
                #    valstr = valstr + '<' + token.value + t2.value + t3.value
                break

        c = readchar()
    if verbose:
        print "multi-line attribute value=", valstr
    return valstr

# read the value of a single-line attribute
# '=' sign has already been read
def get_singleline_value():
    global verbose

    # FIXTHIS - should read to eol directly, bypassing get_token
    # (the current algorithm may alter whitespace)
    first_value = 1
    #token = get_token()
    valstr = ''
    #while token and token.value != '\n':
    valstr = read_string("\n")
        #valstr = valstr + token.whitespace + token.value
        #token = get_token()

    if verbose:
        print "single-line attribute value=", valstr
    return valstr

def get_container_node(node_type, parent):
    global verbose
    if verbose:
        print "get_container_node for node type=", node_type

    # have just read the node type (eg, "option")

    # now read the name of this item
    name = ""
    first_section = 1
    token = get_token()
    while token.value != '>':
        if first_section:
            name = token.value
            first_section = 0
        else:
            name = name + ' ' + token.value
        token = get_token()
        if not token:
            raise SyntaxError, "Missing tag close delimiter"
    if verbose:
        print "creating node, name=", name

    node = node_c(node_type, name, parent)
    
    # all following items, until the closing tag (eg, </option>)
    # are attributes of this item
    token = get_token()
    while token:
        child = process_token(token, node)
        if child:
            if child == 'close_tag':
                break
            if not hasattr(node, "childlist"):
                node.childlist = []
            node.childlist.append(child)
        token = get_token()    
    return node

def add_attr(node, tag, value):
    global verbose

    if verbose:
        print "adding attribute to node"
        print "(", node.name, tag, value,")"

    if not value:
        return

    # add this attribute to the current parent node
    setattr(node, tag, value)
    #raise SyntaxError, "unhandled tag %s" % tag

# processes a token from the current .ecd file
# assigns any created nodes to the indicated parent
# returns a node tree from the tokenized output
# if the token is the start of a container node definition, then
#     a new node is created and it is populated, along with children
#     and the node is returned
# if the token is the start of an attribute, the attribute is
#     added to the parent node
#    and None is returned
# if the token represents a close_tag,
#    then the string "close_tag" is returned.
def process_token(token, parent=None):
    global ifstack, verbose

    # if it's a child, put it on the child list
    # if it's an attribute, make it an attribute of the item
    if verbose:
        print_token(token)

    tag = ''
    tag_value = ''
    if token.value=='<':
        # get item node type (or section tag)
        token=get_token()     #get section type or tag (eg, OPTION)
        if verbose:
            print "found <",token.value

        if token.value in container_tags:
            node = get_container_node(token.value, parent)
            return node
                    
        if token.value in attribute_tags:
            tag = token.value
            if verbose:
                print "processing tag(m)=", tag
            token = get_token()    # read the closing '>'
            if token.value != '>':
                raise SyntaxError, "invalid syntax for tag %s" % tag
            tag_value = get_multiline_value(tag)
            add_attr(parent, tag, tag_value)
            return None

        # check for a close tag
        if token.value[0]=='/' and \
            string.lower(token.value[1:]) in container_tags:
            close_tag = token.value
            # get the closing '>'
            token = get_token()
            if token.value != '>':
                raise SyntaxError, "missing '>' in close tag %s" % close_tag
            if verbose:
                print "close_tag=", close_tag 
            return "close_tag"

    if token.value in attribute_tags:
        # must be a single-line value
        # read '=' and then value
        tag = token.value
        if verbose:
            print "processing tag(1)=", tag
        token = get_token()    # should be '='
        if token.value != '=':
            raise SyntaxError, 'missing "=" for %s tag' % tag
        # build value until eol
        tag_value = get_singleline_value()
        add_attr(parent, tag, tag_value)
        return None

    if token.value=='\n':
        return None
    raise SyntaxError, "fell off the end processing token %s" % token.value

def dump_tree(node, indent=""):
    if node:
        print indent,node
        for attr in node_c.attrlist:
            if hasattr(node, attr):
                print indent,'  ',attr,"=",getattr(node,attr)
        if hasattr(node, "childlist"):
            print indent,'  ','childlist='
            for child in node.childlist:
                if not child:
                    print indent+"    None"
                dump_tree(child, indent+"    ")

####################################################################
#    config file processing    
####################################################################
    
# parse a single ecd file into a node tree
# this routine returns a tuple with the tree (in heirarchical form)
# and a dictionary (flat map) of the nodes from the indicated ECD
def parse_ecd(source_dir, filename, verbose_flag=1):
    global lines, curline, curpos, toklist, ifstack, nodemap
    global verbose
    
    verbose = verbose_flag

    if not len(source_dir):
        source_dir = "."
    #print "Converting ", source_dir+"/"+filename
    lines = read_file(source_dir+"/"+filename)
    if verbose:
        print "lines=",lines
    curline = 0
    toklist = []
    curpos = -1
    nodemap = {}
    token = get_token()
    data = process_token(token)
    return (data, nodemap)

def write_ecd(filename, node):
    file = open(filename,"w")

    indent = ""
    node.write_section(file, indent)
    file.close()


####################################################################
#    utility functions
####################################################################

#
# define some helper functions first.
# getline and getlines collect input from external tools
#
def getline(command):
    file = os.popen(command)
    item = file.readline()
    file.close()
    return item

def getlines(command):
    file = os.popen(command)
    line = file.readline()
    list = []
    while line:
        list.append(line[:-1])
        line = file.readline()
    file.close()
    return list


def usage(exitcode):
        print """\
ecdlib.py - a library of routines to read and process ECD files

Options:
    -h or --help      show this usage help
        -v                show verbose output
    -f <filename>      load the indicated ecd file
                          (default is boa.ecd)
        -d <dir>          use the indicated source directory
              (default is /opt/Embedix/config-data/ecds)
"""
        sys.exit(exitcode)


def test():
    if '-h' in sys.argv or '--help' in sys.argv:
        usage(0)

    verbose_flag = 0
    if '-v' in sys.argv:
        verbose_flag = 1

    ecd_file = "boa.ecd"
    if '-f' in sys.argv:
                try:
                        ecd_file = sys.argv[sys.argv.index('-f')+1]
                except:
                        print "Missing parameter for -f"

    source_dir = "/opt/Embedix/config-data/ecds"
    if '-d' in sys.argv:
                try:
                        source_dir = sys.argv[sys.argv.index('-d')+1]
                except:
                        print "Missing parameter for -d"

    (data, nodemap) = parse_ecd(source_dir, ecd_file, verbose_flag)

    if '-o' in sys.argv:
        print "================== writing ecd back out ==================="
        data.write_node(sys.stdout)

    if verbose:
        print "================== resulting tree ========================="
        dump_tree(data)

if __name__=="__main__":
    test()
