
import sys, os
import curry
import ecdlib
from qt import *
from ecdfrm import ecdfrm
from messaging import stdMsg, errMsg, dbgMsg


class EcdWidget(ecdfrm):
    attributes = [
        "name",
        "prompt",
        "nodetype"
        "default_value",
        "value",
        "type"
        "help",
        "keeplist",
        "provides",
        "build_vars",
        "requires",
        "requiresexpr",
        "license"
        ]
    nodeTypes = ["option", "group", "component", "autovar"]
    
     
    def __init__(self,parent = None, name = None):
        
        
        #set up some icons
        global expanded_icon, closed_icon, enabled_icon, disabled_icon
        expanded_icon = QPixmap(expanded_xpm)
        closed_icon = QPixmap(closed_xpm)
        enabled_icon = QPixmap(enabled_xpm)
        disabled_icon = QPixmap(disabled_xpm)    
        
        ecdfrm.__init__(self,parent,name)       
        #self.nameLineEdit.setReadOnly(1)
        self.modified = 0
        self.modifying = 1      
        
        
        QObject.connect(self.nameLineEdit, SIGNAL("textChanged(const QString&)"), self.setModified)
        QObject.connect(self.promptLineEdit, SIGNAL("textChanged(const QString &)"), curry.curry(self.emitPySignal, PYSIGNAL("promptChanged"))) 
        QObject.connect(self.promptLineEdit, SIGNAL("textChanged(const QString&)"), self.setModified)
        QObject.connect(self.valueLineEdit, SIGNAL("textChanged(const QString&)"), self.setModified)
        QObject.connect(self.defaultValueLineEdit, SIGNAL("textChanged(const QString&)"), self.setModified)
        QObject.connect(self.nodeTypeComboBox, SIGNAL("activated(const QString&)"), curry.curry(self.emitPySignal, PYSIGNAL("nodeTypeChanged")))
        QObject.connect(self.nodeTypeComboBox, SIGNAL("activated(const QString&)"), self.setModified)
        QObject.connect(self.valueTypeComboBox, SIGNAL("activated(const QString&)"), self.setModified)
        QObject.connect(self.multiLineEdit, SIGNAL("textChanged()"), self.setModified)
        QObject.connect(self.listView, SIGNAL("selectionChanged(QListViewItem *)"), self.slotSelectionChanged)
        
        self.radioButtons =(self.helpRadioButton, self.keeplistRadioButton, self.providesRadioButton,
            self.build_varsRadioButton, self.requiresRadioButton, self.requiresexprRadioButton, self.licenseRadioButton)
        self.curriedFunctions = []
        for radioButton in self.radioButtons: 
            
            #Change the class of the radioButton on the fly, this is probably evil
            radioButton.__class__ = ColorableRadioButton
            radioButton.bold = 0
            
            QObject.connect(radioButton, SIGNAL("clicked()"), curry.curry(self.slotRadioButtonClicked, radioButton.name()))
        self.helpRadioButton.setChecked(1)
        dbgMsg("ecd widget init function finished")
        self.show()
        
    def emitPySignal(self, pySignal, *args):
        self.emit(pySignal, args)
    
    def setListItemName(self, newName):
        self.listView.currentItem().setText(0, newName)
    def setListItemType(self, newType):
        self.listView.currentItem().setText(1, newType)

    
    #getter/setter methods for the node tab
    def getNodeName(self):
        return str(self.nameLineEdit.text())        
    def setNodeName(self, name, modifying=0):
        oldModifying = self.modifying
        self.modifying = modifying    
        self.nameLineEdit.setText(str(name))
        self.modifying = oldModifying
        
    def getPrompt(self):
        return str(self.promptLineEdit.text())
    def setPrompt(self, prompt, modifying=0):
        oldModifying = self.modifying
        self.modifying = modifying    
        self.promptLineEdit.setText(str(prompt))
        self.modifying = oldModifying    
    
    def getNodeType(self):
        return str(self.nodeTypeComboBox.currentText())
    def setNodeType(self, nodeType, modifying=0):
        oldModifying = self.modifying
        self.modifying = modifying       
        nodeType = nodeType.lower()
        for i in range(self.nodeTypeComboBox.count()):
            if str(self.nodeTypeComboBox.text(i)) == nodeType:
                self.nodeTypeComboBox.setCurrentItem(i)
                self.modifying = oldModifying
                return
        errMsg(nodeType + " is not a valid nodetype!")
        self.modifying = oldModifying

    #getter/setter methods for value tab
    def getValueType(self):
        return str(self.valueTypeComboBox.currentText())
    def setValueType(self, valueType, modifying=0):
        oldModifying = self.modifying
        self.modifying = modifying 
        valueType = valueType.lower()
        if valueType.strip() == "":
            valueType = "bool"
        for i in range(self.valueTypeComboBox.count()):
            if str(self.valueTypeComboBox.text(i)) == valueType:
                self.valueTypeComboBox.setCurrentItem(i)
                self.modifying = oldModifying
                return
        errMsg(valueType + " is not a valid valuetype!")       
        self.modifying = oldModifying
            
    def getValue(self):
        return str(self.valueLineEdit.text())
    def setValue(self, value, modifying=0):
        oldModifying = self.modifying
        self.modifying = modifying 
        self.valueLineEdit.setText(value)
        self.modifying = oldModifying
       
    def getDefaultValue(self):
        return str(self.defaultValueLineEdit.text())
    def setDefaultValue(self, defaultValue, modifying=0):
        oldModifying = self.modifying
        self.modifying = modifying 
        self.defaultValueLineEdit.setText(defaultValue)
        self.modifying = oldModifying
        
    def getMultiLineEdit(self):
        return str(self.multiLineEdit.text())
    def setMultiLineEdit(self, text, modifying=0):
        oldModifying = self.modifying
        self.modifying = modifying 
        self.multiLineEdit.setText(text)
        self.modifying = oldModifying
        
    def setRadioButtonBold(self, field, bold):    
        radioButton = self.__dict__[field + "RadioButton"]
        radioButton.bold = bold
        radioButton.repaint()
        
    #getter/setter methods for the multiline edit field
       
    def slotRadioButtonClicked(self, radioButtonName):
        dbgMsg("radio button %s was selected" % radioButtonName)
        self.emit(PYSIGNAL("radioButtonClicked"), (radioButtonName,))
        
    def slotSelectionChanged(self, node):
        dbgMsg("Node %s selected" % node.name)
        self.emit(PYSIGNAL("selectionChanged"), (node,))
        
    def getCurrentRadioButton(self):
        return self.radioButtonGroup.selected().name()    
            
    def setModified(self, modified = 1):
        #if we have set our mode to not do modifying then don't send the signal!
        if self.modifying == 0:
            return
        if modified:
            self.modified = 1
        else:
            self.modified = modified
        self.emit(PYSIGNAL("modified"), (self.modified,))
            
        dbgMsg("modified set to %i" % self.modified)
        

################ here are some fun pixmaps #####################        
        
expanded_xpm = [
"12 7 18 1",
"       c None",
".      c #000000",
"+      c #B8C2AE",
"@      c #9DAB8F",
"#      c #82946F",
"$      c #677D50",
"%      c #4C6631",
"&      c #314E12",
"*      c #214000",
"=      c #888888",
"-      c #BBBBBB",
";      c #879976",
">      c #6C8157",
",      c #516A37",
"'      c #365318",
")      c #576F3E",
"!      c #3C581F",
"~      c #264506",
"........... ",
" .+@#$%&*.=-",
"  .;>,'*.=- ",
"   .)!*.=-  ",
"    .~.=-   ",
"     .=-    ",
"      -     "]

closed_xpm = [
"7 12 18 1",
"       c None",
".      c #000000",
"+      c #CAD2C2",
"@      c #AAB69D",
"#      c #96A587",
"$      c #899A77",
"%      c #758961",
"&      c #62784B",
"*      c #687E52",
"=      c #556D3B",
"-      c #415C25",
";      c #2E4C0F",
">      c #48622C",
",      c #345116",
"'      c #214000",
")      c #888888",
"!      c #BBBBBB",
"~      c #274607",
".      ",
"..     ",
".+.    ",
".@#.   ",
".$%&.  ",
".*=-;. ",
".>,'.)!",
".~'.)! ",
".'.)!  ",
"..)!   ",
".)!    ",
" !     "]

disabled_xpm = [
"16 11 4 1",
"       c None",
".      c #D7D7D7",
"+      c #B7B7B7",
"@      c #777777",
"                ",
"                ",
"                ",
"...........     ",
".+++++++++@     ",
".+++++++++@     ",
".+++++++++@     ",
".+++++++++@     ",
"@@@@@@@@@@@     ",
"                ",
"                "]

enabled_xpm = [
"16 13 90 1",
"       c None",
".      c #001607",
"+      c #003611",
"@      c #004315",
"#      c #003310",
"$      c #001306",
"%      c #004516",
"&      c #00A134",
"*      c #00DF49",
"=      c #00F04E",
"-      c #00D144",
";      c #008D2E",
">      c #003811",
",      c #00892C",
"'      c #00D345",
")      c #00F751",
"!      c #00EE4E",
"~      c #00E44A",
"{      c #00DE49",
"]      c #00D646",
"^      c #00AB38",
"/      c #006A22",
"(      c #004215",
"_      c #00CD43",
":      c #00F24F",
"<      c #00E74C",
"[      c #00D947",
"}      c #00D044",
"|      c #00CB42",
"1      c #00C841",
"2      c #009F34",
"3      c #00300F",
"4      c #001506",
"5      c #009931",
"6      c #00EE4D",
"7      c #00E24A",
"8      c #00CE43",
"9      c #00C13F",
"0      c #00BD3E",
"a      c #00BC3E",
"b      c #00BC3D",
"c      c #00B33A",
"d      c #006821",
"e      c #000D04",
"f      c #00C03F",
"g      c #00B83C",
"h      c #00B73B",
"i      c #00B039",
"j      c #00912F",
"k      c #001F09",
"l      c #004015",
"m      c #00C942",
"n      c #00B83B",
"o      c #009E34",
"p      c #00250B",
"q      c #00BB3D",
"r      c #00CF44",
"s      c #00C540",
"t      c #00BA3D",
"u      c #00B63B",
"v      c #00AF39",
"w      c #008C2D",
"x      c #001D08",
"y      c #001105",
"z      c #007B28",
"A      c #00C23F",
"B      c #00B93D",
"C      c #00B53B",
"D      c #00A937",
"E      c #00601F",
"F      c #000B03",
"G      c #002F0F",
"H      c #009631",
"I      c #00B73C",
"J      c #00852B",
"K      c #00260C",
"L      c #00591D",
"M      c #00A737",
"N      c #00AA37",
"O      c #00AC38",
"P      c #00A636",
"Q      c #00842B",
"R      c #004D19",
"S      c #00290D",
"T      c #005B1D",
"U      c #009430",
"V      c #000A03",
"W      c #001807",
"X      c #001D09",
"Y      c #001707",
"    .+@#$       ",
"   %&*=-;>      ",
"  ,')!~{]^/     ",
" (_:<[}_|123    ",
"4567890abbcde   ",
"+87'fghghhijk   ",
"l[]mbnnghhiop   ",
"3qrsthhghuvwx   ",
"yzA9BhhghCDEF   ",
" GHgICuuCvJK    ",
"  L;MNOOPQR     ",
"   STQUQTp      ",
"    VWXYV       "]



ecd_tags=["name", "nodetype", "type", "default_value", "prompt", "license", "help", "keeplist", 
    "min_dynamic_size", "provides", "build_requires", "build_vars",
    "requires", "requiresexpr", "srpm", "specpatch",
    "startup_time", "static_size", "storage_size",
    "value", "trideps", "choicelist", "if", "conflicts",
    "transient", "range", "build_time"]
    
class ColorableRadioButton(QRadioButton):
    i = 0
    def __init__(self, parent, name):
        QRadioButton.__init__(self, parent, name)
        self.bold = 0
        
    def setBold(self, bold):
        if not bold in [0, 1]:
            errMsg("received bad bols value %s" % str(bold))
            return
        self.bold = bold
        self.repaint()

    def bold(self):
        return self.bold

    def drawButtonLabel(self, painter):
        font = painter.font()
        font.setBold(self.bold)
        QRadioButton.drawButtonLabel(self, painter)    
    
class EcdNodeViewItem(QListViewItem):
    def __init__(self, name, text, nodetype, parent):
        QListViewItem.__init__(self, parent)
        self.children = []
        self.name = name
        self.setText(0, text)
        self.setText(1, nodetype)
        self.lineoGreen = QColor(33, 65, 0)
        self.setPixmap(0, enabled_icon)
    
    #this is taken directly from TW code for eqtlistviewitem
    def paintBranches(self, p, cg, w, y, h, s=None):
        p.fillRect(0, 0, w, h, QBrush(cg.base()))
        child = self.firstChild()
        linebot = 0
        while child != None and y + child.height() <= 0:
            y += child.totalHeight()
            child = child.nextSibling()
        bx = w/2
        while child != None and y < h:
            linebot = y + child.height() / 2
            if (child.isExpandable() or child.childCount() > 0) and child.height() > 0:
                p.setPen(self.lineoGreen.dark())
                if child.isOpen():
                    p.drawPixmap(bx-4, linebot-4, expanded_icon)
                else:
                    p.drawPixmap(bx-4, linebot-4, closed_icon)
                p.setBrush(p.NoBrush)
            y += child.totalHeight()
            child = child.nextSibling()

    def addChild(self, child):
        self.children.append(child)

    def expandTree(self):
        self.setOpen(1)
        node = self.firstChild()
        while node:
            node.expandTree()
            node = node.nextSibling()
    
            
        
class EcdObject(QObject):
    def __init__(self):
        QObject.__init__(self)
        self.node_map = {}
        
    def readEcd(self, filename):
        self.filename = filename
        (dir, file) = os.path.split(self.filename)
        (self.root, node_map) = ecdlib.parse_ecd(dir, file, verbose_flag = 0)
        self.node_map = node_map.copy()

    def getRootName(self):
        return self.root.name

    def hasNode(self, ecd_name):
        return self.node_map.has_key(ecd_name)

    def getComponentNodeName(self):
        for key in self.node_map.keys():
            if self.node_map[key].nodetype.lower() == "component":
                return key
        return ""

    def writeEcdNode(self, node_name, fd):
        if not self.node_map.has_key(node_name):
            dbgMsg(5, "could not find node %s" % node_name)
            return
        node = self.node_map[node_name]
        node.write_node(fd)

    def writeEcd(self, fd):
        self.root.write_node(fd)
    
    def getEcdNodeField(self, node_name, field_name):
        if not self.node_map.has_key(node_name):
            dbgMsg(5, "could not find node %s" % node_name)
            return ""
        if not field_name in ecd_tags:
            dbgMsg(5, "%s is not a valid ecd field" % field_name)
            return ""
        node = self.node_map[node_name]
        #print "ECDOBject::entry field %s to value %s for node %s" % (field_name, value, node_name)
        return getattr(node, field_name, "")
    
    def setEcdNodeField(self, node_name, field_name, value):
        if not self.node_map.has_key(node_name):
            dbgMsg("could not find node %s" % node_name)
            return ""
        if not field_name in ecd_tags:
            dbgMsg("%s is not a valid ecd field" % field_name)
            return ""
        #print "ECDOBject::entry field %s to value %s for node %s" % (field_name, value, node_name)
        node = self.node_map[node_name]
        old_value = getattr(node, field_name, "")
        if old_value != value:
            self.emit(PYSIGNAL("ModifiedSignal"), ("ECD",1))
        setattr(node, field_name, value)

    def changeNodeName(self, old_name, new_name):
        self.node_map[new_name] = self.node_map[old_name]
        del self.node_map[old_name]
        self.node_map[new_name].name = new_name

    def removeNode(self, node_name):
        if not self.node_map.has_key(node_name):
            dbgMsg(5, "could not find node %s" % node_name)
            return
        
        node = self.node_map[node_name]
        #if the node has a parent then remove itself from the childlist
        if node.parent:
            node.parent.childlist.remove(node)
        #remove the node from the dictionary
        del self.node_map[node_name]

    def getEcdNodeChildren(self, node_name):
        childList = []
        if not self.node_map.has_key(node_name):
            dbgMsg(5, "could not find node %s" % node_name)
            return childList
        node = self.node_map[node_name]
        for child in node.childlist:
                childList.append(child.name)
        return childList

    def createNode(self, parent_name, node_name, node_type):
        if parent_name == "root":
            node = ecdlib.node_c(node_type, node_name)
            self.root = node
        else:
            if not self.node_map.has_key(parent_name):
                dbgMsg("could not find node %s" % parent_name)
                return
            parent = self.node_map[parent_name]
            node = ecdlib.node_c(node_type, node_name, parent)
            parent.childlist.append(node)
        self.node_map[node_name] = node
        


class EcdController(QObject):
    def __init__(self, ecdWidget):
        QObject.__init__(self, None, "EcdController")
        
        self.ecdWidget = ecdWidget
        QObject.connect(ecdWidget, PYSIGNAL("selectionChanged"), self.newNodeSelected)
        QObject.connect(ecdWidget, PYSIGNAL("radioButtonClicked"), self.radioButtonClicked)
        QObject.connect(ecdWidget, PYSIGNAL("promptChanged"), self.promptChanged)
        QObject.connect(ecdWidget, PYSIGNAL("nodeTypeChanged"), self.nodeTypeChanged)
        
        
        self.ecdNodeItemDict = {}
        self.oldNode = None
        self.oldField = None
        self.ecdWidget.setEnabled(0)
        
    def clearTree(self):
        self.ecdWidget.listView.clear()
        
    def readEcd(self, filename):
        #FIXTHIS - check for existence of file
        #print "reading file", filename
        self.clearTree()
        self.embedixNode = EcdNodeViewItem("Embedix", "Embedix", "group", self.ecdWidget.listView)
        self.ecdWidget.setEnabled(1)
        self.ecdObject = EcdObject()
        self.ecdObject.readEcd(filename)
        self.ecdNodeItemDict.clear()
        
        root_name = self.ecdObject.getRootName()
        self.addChildNodes(self.embedixNode, [root_name])
        self.embedixNode.expandTree()
            
    def writeEcd(self, filename):
        fd = open(filename, "w")
        self.ecdObject.writeEcd(fd)    
    
    def addChildNodes(self, parent, childrenNames):
        #print "in AddChildNodes, parent is", parent, "children are", children_names
        for childName in childrenNames:
            text = self.ecdObject.getEcdNodeField(childName, "prompt")
            if text.strip() == "":
                text = self.ecdObject.getEcdNodeField(childName, "name")
            nodeType = self.ecdObject.getEcdNodeField(childName, "nodetype")
            self.ecdNodeItemDict[childName] = EcdNodeViewItem(childName, text, nodeType, parent)
            parent.addChild(self.ecdNodeItemDict[childName])
            self.addChildNodes(self.ecdNodeItemDict[childName], self.ecdObject.getEcdNodeChildren(childName))
    
    def promptChanged(self, newPrompt):
        newPrompt = str(newPrompt).strip()
        #don't set the node attribute from here because it creates sync problems!
        #self.ecdObject.setEcdNodeField(self.oldNode.name, "prompt", newPrompt)
        self.ecdWidget.setListItemName(newPrompt)
        
    def nodeTypeChanged(self, newType):
        newType = str(newType).strip()
        self.ecdWidget.setListItemType(newType)
    
    def radioButtonClicked(self, radioButtonName):
        dbgMsg("button %s was selected" % radioButtonName)   
        if self.oldField != None:
            self.ecdObject.setEcdNodeField(self.oldNode.name, self.oldField, self.ecdWidget.getMultiLineEdit().strip())
        
        field = radioButtonName[:-len("RadioButton")]
        self.ecdWidget.setMultiLineEdit(self.ecdObject.getEcdNodeField(self.oldNode.name, field).strip())
        self.oldField = field
    
    def newNodeSelected(self, newNode):
        dbgMsg("New Node Selected", newNode.name)
        radioButtonName = self.ecdWidget.getCurrentRadioButton()
        dbgMsg("%s is the current radio button" % radioButtonName)
        field = radioButtonName[:-len("RadioButton")] #get keeplist from keeplistRadioButton or help from helpRadioButton
        dbgMsg("field is", field)
        
        if self.oldNode != None and self.oldNode.name != "Embedix":
            name = self.oldNode.name
            self.ecdObject.setEcdNodeField(name, "prompt", self.ecdWidget.getPrompt())
            self.ecdObject.setEcdNodeField(name, "nodetype", self.ecdWidget.getNodeType())
            
            self.ecdObject.setEcdNodeField(name, "default_value", self.ecdWidget.getDefaultValue())
            self.ecdObject.setEcdNodeField(name, "value", self.ecdWidget.getValue())
            self.ecdObject.setEcdNodeField(name, "type", self.ecdWidget.getValueType())
            
            self.ecdObject.setEcdNodeField(name, field, self.ecdWidget.getMultiLineEdit())
            
        
        name = newNode.name
        if name.strip() == "Embedix":
            self.ecdWidget.setNodeName(newNode.name)
            self.ecdWidget.setPrompt("Embedix")
            self.ecdWidget.setNodeType("group")
        
            self.ecdWidget.setDefaultValue("")
            self.ecdWidget.setValue("")
            self.ecdWidget.setValueType("bool")
        
            self.ecdWidget.setMultiLineEdit("")
            for field in ["help", "keeplist", "provides", "build_vars", "requires", "requiresexpr", "license"]:
                self.ecdWidget.setRadioButtonBold(field, 0)
         
        else:
            self.ecdWidget.setNodeName(name)
            prompt = self.ecdObject.getEcdNodeField(name, "prompt")
            if prompt == "":
                prompt = name
            self.ecdWidget.setPrompt(prompt)
            self.ecdWidget.setNodeType(self.ecdObject.getEcdNodeField(name, "nodetype"))
        
            self.ecdWidget.setDefaultValue(self.ecdObject.getEcdNodeField(name, "default_value"))
            self.ecdWidget.setValue(self.ecdObject.getEcdNodeField(name, "value"))
            self.ecdWidget.setValueType(self.ecdObject.getEcdNodeField(name, "type"))
            
            self.ecdWidget.setMultiLineEdit(self.ecdObject.getEcdNodeField(name, field).strip())
            
            for field in ["help", "keeplist", "provides", "build_vars", "requires", "requiresexpr", "license"]:
                if self.ecdObject.getEcdNodeField(name, field).strip() == "":
                    self.ecdWidget.setRadioButtonBold(field, 0)
                else:
                    self.ecdWidget.setRadioButtonBold(field, 1)
        self.oldNode = newNode       
    
    
if __name__ == '__main__':
    a = QApplication(sys.argv)
    QObject.connect(a,SIGNAL('lastWindowClosed()'),a,SLOT('quit()'))
    w = EcdWidget()
    ecdController = EcdController(w)
    ecdController.readEcd("make_deb.ecd")
    a.setMainWidget(w)
    w.show()
    a.exec_loop()            
