#this is a module used for messaging.  It allows multiple classes
#to handle various types of

import sys, string, exceptions

class MessagingException(exceptions.Exception):
    def __init__(self, args=None):
        self.args = args

class MessageHandler:
    def handleStdMsg(self, msg):
        return -1
    def handleErrMsg(self, msg):
        return -1
    def handleDbgMsg(self, msg):
        return -1

class defaultMessageHandler(MessageHandler):
    def handleStdMsg(self, msg):
        sys.stdout.write(msg + "\n")
    def handleErrMsg(self, msg):
        sys.stderr.write(msg + "\n")
    def handleDbgMsg(self, msg):
        sys.stdout.write(msg + "\n")

try:
    from qt import *
    qtImported = 1
except:
    qtImported = 0

if qtImported:
        class QMessageHandler(QMultiLineEdit, MessageHandler):
                def __init__(*args):
                        apply(QMultiLineEdit.__init__, args)
                        self = args[0]
                        self.setReadOnly(1)
                def handleStdMsg(self, msg):
                        self.insertLine(msg)
                def handleErrMsg(self, msg):
                        self.insertLine(msg)
                def handleDbgMsg(self, msg):
                        self.insertLine(msg)


_stdMessageHandlers = []
_errMessageHandlers = []
_dbgMessageHandlers = []

#call this with the handler as the first argument and the types
#of messages to handle
def registerMessageHandler(handler, *types):
    for msgType in types:
        if msgType == "std":
            _stdMessageHandlers.append(handler)
        elif msgType == "err":
            _errMessageHandlers.append(handler)
        elif msgType == "dbg":
            _dbgMessageHandlers.append(handler)
        else:
            raise MessagingException, msgType + " is not a valid message type to register."

#send this message to all handlers of std messages
def stdMsg(*args):
    s = string.join(map(str, args), " ")
    for handler in _stdMessageHandlers:
        handler.handleStdMsg(s)

#send this message to all handlers of error messages
def errMsg(*args):
    s = "Error in "
    f = None
    
    #throw an exception to get access to the traceback
    #which we can get the stack frame from.  Then we get
    #access to the caller's stack and variables to get the 
    #callers method and class names!
    try:
        1/0
    except Exception, e:
        f = sys.exc_info()[2].tb_frame.f_back
               
    #f = sys._getframe(1)
    obj = f.f_locals.get("self", None)
    functionName = f.f_code.co_name
    if obj:
        s += obj.__class__.__name__ + "::" + functionName
    else:
        s += functionName
    s += " : " + string.join(map(str, args), " ")
    for handler in _errMessageHandlers:
        handler.handleErrMsg(s)

def dbgMsg(*args):
    s = ""
    f = None
    
    #throw an exception to get access to the traceback
    #which we can get the stack frame from.  Then we get
    #access to the caller's stack and variables to get the 
    #callers method and class names!
    try:
        1/0
    except Exception, e:
        f = sys.exc_info()[2].tb_frame.f_back
               
        
    #f = sys._getframe(1)
    obj = f.f_locals.get("self", None)
    functionName = f.f_code.co_name
    
    if obj:
        s += obj.__class__.__name__ + "::" + functionName
    else:
        s += functionName
    s += " : " + string.join(map(str, args), " ")
    for handler in _dbgMessageHandlers:
        handler.handleDbgMsg(s)


registerMessageHandler(defaultMessageHandler(), "std", "err", "dbg")

#the rest is simply here for testing
def testMsg():
        print "in testing function"
        dbgMsg("in testing function, something went bad!")
        errMsg("in testing function, something went bad!")

class TestClass:
        def test(self):
                print "in class testing function"
                dbgMsg("in class testing function, something went bad!")
                errMsg("This is an Error!")

def test():
    sendTestMessages()

timerTimes = 0
def qtTest():
    global timer
    qApp = QApplication(sys.argv)
    qmessager = QMessageHandler()
    registerMessageHandler(qmessager, "std", "err", "dbg")
    qmessager.setName("Qt Message Handler")
    qApp.setMainWidget(qmessager)
    qmessager.show()
    timer = QTimer()
    timer.start(500)
    QObject.connect(timer, SIGNAL("timeout()"), sendQtMessage)
    qApp.exec_loop()

def sendQtMessage():
    sendTestMessages()
    global timerTimes
    timerTimes += 1
    if timerTimes == 10:
        timer.stop()


def sendTestMessages():
    testMsg()
    TestClass().test()
    errMsg("This is an error!")
    stdMsg("This is a", "standard", "message", 3, "times")
    dbgMsg("This is a debug message")


if __name__ == "__main__":
    if qtImported:
        qtTest()
    else:
        test()







